=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2008-06-07 22:15:25 +0000
+++ bzrlib/branch.py	2008-06-11 07:52:25 +0000
@@ -528,7 +528,7 @@
         return history[revno - 1]
 
     def pull(self, source, overwrite=False, stop_revision=None,
-             possible_transports=None):
+             possible_transports=None, _override_hook_target=None):
         """Mirror source into this branch.
 
         This branch is considered to be 'local', having low latency.
@@ -1557,7 +1557,8 @@
 
     @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
-             _hook_master=None, run_hooks=True, possible_transports=None):
+             _hook_master=None, run_hooks=True, possible_transports=None,
+             _override_hook_target=None):
         """See Branch.pull.
 
         :param _hook_master: Private parameter - set the branch to 
@@ -1568,7 +1569,10 @@
         """
         result = PullResult()
         result.source_branch = source
-        result.target_branch = self
+        if _override_hook_target is None:
+            result.target_branch = self
+        else:
+            result.target_branch = _override_hook_target
         source.lock_read()
         try:
             # We assume that during 'pull' the local repository is closer than
@@ -1581,9 +1585,9 @@
             result.new_revno, result.new_revid = self.last_revision_info()
             if _hook_master:
                 result.master_branch = _hook_master
-                result.local_branch = self
+                result.local_branch = result.target_branch
             else:
-                result.master_branch = self
+                result.master_branch = result.target_branch
                 result.local_branch = None
             if run_hooks:
                 for hook in Branch.hooks['post_pull']:
@@ -1753,7 +1757,8 @@
         
     @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
-             run_hooks=True, possible_transports=None):
+             run_hooks=True, possible_transports=None,
+             _override_hook_target=None):
         """Pull from source into self, updating my master if any.
         
         :param run_hooks: Private parameter - if false, this branch
@@ -1773,7 +1778,8 @@
                     run_hooks=False)
             return super(BzrBranch5, self).pull(source, overwrite,
                 stop_revision, _hook_master=master_branch,
-                run_hooks=run_hooks)
+                run_hooks=run_hooks,
+                _override_hook_target=_override_hook_target)
         finally:
             if master_branch:
                 master_branch.unlock()

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2008-06-10 07:37:59 +0000
+++ bzrlib/remote.py	2008-06-11 07:52:25 +0000
@@ -1518,7 +1518,7 @@
         self._ensure_real()
         return self._real_branch.pull(
             source, overwrite=overwrite, stop_revision=stop_revision,
-            **kwargs)
+            _override_hook_target=self, **kwargs)
 
     @needs_read_lock
     def push(self, target, overwrite=False, stop_revision=None):

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2008-05-28 22:30:48 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2008-06-11 07:52:25 +0000
@@ -165,7 +165,7 @@
         wt_a.add(['one'])
         wt_a.commit('commit one', rev_id='1')
 
-        branch_b = wt_a.bzrdir.sprout('b', revision_id='1').open_branch()
+        branch_b = wt_a.branch.bzrdir.sprout('b', revision_id='1').open_branch()
         self.assertEqual(wt_a.branch.base, branch_b.get_parent())
         return branch_b
 

=== modified file 'bzrlib/tests/branch_implementations/test_check.py'
--- bzrlib/tests/branch_implementations/test_check.py	2008-04-29 17:01:24 +0000
+++ bzrlib/tests/branch_implementations/test_check.py	2008-06-11 07:52:25 +0000
@@ -43,6 +43,11 @@
         # Now, try to set an invalid history
         try:
             tree.branch.set_revision_history([r1, r2b, r5])
+            if tree.branch.last_revision_info() != (3, r5):
+                # RemoteBranch silently corrects an impossible revision
+                # history given to set_revision_history.  It can be tricked
+                # with set_last_revision_info though.
+                tree.branch.set_last_revision_info(3, r5)
         except errors.NotLefthandHistory:
             # Branch5 allows set_revision_history to be wrong
             # Branch6 raises NotLefthandHistory, but we can force bogus stuff

=== modified file 'bzrlib/tests/branch_implementations/test_reconcile.py'
--- bzrlib/tests/branch_implementations/test_reconcile.py	2008-04-29 16:43:14 +0000
+++ bzrlib/tests/branch_implementations/test_reconcile.py	2008-06-11 07:52:25 +0000
@@ -43,6 +43,11 @@
         # Now, try to set an invalid history
         try:
             tree.branch.set_revision_history([r1, r2b, r5])
+            if tree.branch.last_revision_info() != (3, r5):
+                # RemoteBranch silently corrects an impossible revision
+                # history given to set_revision_history.  It can be tricked
+                # with set_last_revision_info though.
+                tree.branch.set_last_revision_info(3, r5)
         except errors.NotLefthandHistory:
             # Branch5 allows set_revision_history to be wrong
             # Branch6 raises NotLefthandHistory, but we can force bogus stuff

